package cn.urstudio.cn.urhot.controller;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.net.URLEncodeUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import cn.urstudio.cn.urhot.bean.Result;
import cn.urstudio.cn.urhot.utils.RssUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author tiger
 * {@code @date} 2022年 05月25日 11:01:12
 */
@RequestMapping("/rss")
@RestController
@CrossOrigin(value = "*")
@Slf4j
public class PublicController {

    @Value("${data.path.cache}")
    private String cachePath;

    @Value("${data.path.keywords}")
    private String keywordsPath;

    @Value("${data.rsshub}")
    private String rssHubUrl;

    /**
     *
     * @param cacheEnable
     * @return
     */
    @GetMapping("")
    public Result list(@RequestParam(name = "cache", defaultValue = "true", required = false) boolean cacheEnable) {
        Result result = new Result();
        JSONArray twitterKeywordsArray = JSONUtil.parseArray(FileUtil.readUtf8String(keywordsPath + "twitter_keywords.json"));
        JSONArray twitterUserArray = JSONUtil.parseArray(FileUtil.readUtf8String(keywordsPath + "twitter_user.json"));
        JSONArray mediumUserArray = JSONUtil.parseArray(FileUtil.readUtf8String(keywordsPath + "medium_user.json"));


        List<Map<String, Object>> resultList = new ArrayList<>();
        for (int i = 0; i < twitterKeywordsArray.size(); i++) {
            //获取twitter关键词
            String twitterKeyword = URLEncodeUtil.encode(twitterKeywordsArray.getJSONObject(i).getStr("name"));
            String url = rssHubUrl + "/twitter/keyword/" + twitterKeyword + "?readable=true&authorNameBold=true&widthOfPics=50&heightOfPics=50&limit=100";
            try {
                Map<String, Object> resultData = RssUtils.load(url, cacheEnable, cachePath, "twitter_keyword", twitterKeyword);
                resultList.add(resultData);
            } catch (Exception e) {
                e.printStackTrace();
            }

        }

        for (int i = 0; i < twitterUserArray.size(); i++) {
            String twitterUser = twitterUserArray.getJSONObject(i).getStr("name");
            String url = rssHubUrl + "/twitter/user/" + twitterUser + "?readable=true&authorNameBold=true&widthOfPics=50&heightOfPics=50&limit=100&excludeReplies=true&includeRts=false";
            try {
                Map<String, Object> resultData = RssUtils.load(url, cacheEnable, cachePath, "twitter_user", twitterUser);
                resultList.add(resultData);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        for (int i = 0; i < mediumUserArray.size(); i++) {
            String mediumUser = mediumUserArray.getJSONObject(i).getStr("name");
            String url = "https://medium.com/feed/" + mediumUser;
            try {
                Map<String, Object> resultData = RssUtils.load(url, cacheEnable, cachePath, "medium", mediumUser);
                resultList.add(resultData);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        result.setCode(200);
        result.setMessage("success");
        result.setData(resultList);

        return result;
    }

    /**
     *
     * @param type
     * @param name
     * @return
     */
    @GetMapping("/{type}/{name}")
    public Result list(@PathVariable String type, @PathVariable String name) {
        String[] types = new String[]{"twitter_keyword", "twitter_user", "medium"};
        if (!ArrayUtil.contains(types, type)) {
            return Result.builder()
                    .code(500)
                    .message("Server Runtime Error: type invalid")
                    .build();
        }

        String url = "";
        switch (type) {
            case "twitter_keyword":

                url = rssHubUrl + "/twitter/keyword/" + URLEncodeUtil.encode(name) + "?readable=true&authorNameBold=true&widthOfPics=50&heightOfPics=50&limit=100";
                break;
            case "twitter_user":
                url = rssHubUrl + "/twitter/user/" + name + "?readable=true&authorNameBold=true&widthOfPics=50&heightOfPics=50&limit=100&excludeReplies=true&includeRts=false";

                break;
            case "medium":
                url = "https://medium.com/feed/" + name;
                break;
            default:
        }
        try {
            Map<String, Object> resultData = RssUtils.load(url, false, cachePath, type, name);
            return Result.builder()
                    .code(200)
                    .message("success")
                    .data(resultData)
                    .build();
        } catch (Exception e) {
            e.printStackTrace();
            return Result.builder()
                    .code(501)
                    .message(e.getMessage())
                    .build();
        }
    }
}
