package cn.urstudio.cn.urhot.utils;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.crypto.digest.MD5;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.rometools.rome.feed.synd.SyndFeed;
import com.rometools.rome.io.SyndFeedInput;
import com.rometools.rome.io.XmlReader;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.io.File;
import java.net.URL;
import java.util.*;

/**
 * @author tiger
 * @date 2022年 05月24日 17:56:16
 */
@Slf4j
@Component
public class RssUtils {


    /**
     * @param url
     * @param cache
     * @param cachePath
     * @param type
     * @return
     * @throws Exception
     */
    public static Map<String, Object> load(String url, boolean cache, String cachePath, String type, String name) throws Exception {

        File cacheFile = new File(cachePath + "feed_" + MD5.create().digestHex(url) + ".json");
        String fileData;
        //启用缓存，缓存文件存在，且文件修改时间在半小时内直接返回文件内容；或者，禁用缓存时，如果文件修改时间在五分钟内，也直接返回文件内容
        boolean flag = (cache && cacheFile.exists() && System.currentTimeMillis() - cacheFile.lastModified() < 60 * 60 * 3 * 1000) || (!cache && cacheFile.exists()  && System.currentTimeMillis() - cacheFile.lastModified() < 60 * 5 * 1000);
        if (flag) {
            log.info("load from cache");
            fileData = FileUtil.readUtf8String(cacheFile.getAbsolutePath());

        }
        //否则，直接获取RSS订阅并存入缓存文件
        else {
            log.info("load from remote");
            SyndFeedInput input = new SyndFeedInput();
            SyndFeed feed = input.build(new XmlReader(new URL(url)));

            fileData = JSONUtil.toJsonStr(feed);
            FileUtil.writeUtf8String(fileData, cacheFile);
        }

        JSONObject feedJsonObj = JSONUtil.parseObj(fileData);
        //title
        //data [{"title":"","author":"","link":"","time":""},{"title":"","author":"","link":"","time":""}]
        Map<String, Object> resultData = new HashMap<>();
        resultData.put("title", feedJsonObj.getStr("title"));
        List<Map<String, Object>> entrieDataList = new ArrayList<>();
        JSONArray entriesArr = feedJsonObj.getJSONArray("entries");
        switch (type) {
            case "twitter_user":
            case "twitter_keyword":
                for (int i = 0; i < entriesArr.size(); i++) {
                    JSONObject entry = entriesArr.getJSONObject(i);
                    Map<String, Object> entryData = new HashMap<>();
                    try {
                        entryData.put("title", entry.getJSONObject("description").getStr("value"));
                    }catch (NullPointerException e){
                        entryData.put("title", entry.getJSONArray("contents").getJSONObject(0).getStr("value"));
                    }
                    try {
                        entryData.put("author", entry.getJSONArray("modules").getJSONObject(0).getStr("creator"));
                    }catch (NullPointerException e){
                        entryData.put("author", entry.getJSONArray("authors").getJSONObject(0).getStr("name"));
                    }
                    try{
                        entryData.put("time", DateUtil.format(new Date(entry.getJSONArray("modules").getJSONObject(0).getLong("date")), "yyyy-MM-dd hh:mm:ss"));
                    }catch (NullPointerException e){
                        Long longDate = entry.getLong("updatedDate");
                        entryData.put("time", DateUtil.format(new Date(longDate), "yyyy-MM-dd hh:mm:ss"));
                    }
                    entryData.put("link", entry.getStr("link"));


                    entrieDataList.add(entryData);
                }
                break;
            case "medium":
                for (int i = 0; i < entriesArr.size(); i++) {
                    JSONObject entry = entriesArr.getJSONObject(i);
                    Map<String, Object> entryData = new HashMap<>();
                    entryData.put("title", entry.getStr("title"));
                    entryData.put("author", entry.getJSONArray("modules").getJSONObject(0).getStr("creator"));
                    entryData.put("link", entry.getStr("link"));
                    entryData.put("time", DateUtil.format(new Date(entry.getJSONArray("modules").getJSONObject(0).getLong("date")), "yyyy-MM-dd hh:mm:ss"));
                    entrieDataList.add(entryData);
                }
                break;
            default:
        }
        resultData.put("type", type);
        resultData.put("name", name);
        resultData.put("data", entrieDataList);
        return resultData;
    }
}
